{ pkgs, nixpkgs, ... }:
let
  # you know the password
  passwordHash = "$6$rounds=656000$Z6W8AmLKe67OfTb8$1SRgfsBQWwpDz4By4KLTiIJZdQiAaDXsNq3nOPU0n/nD7FSFaRiaaZhcD8OuHUEQtzDQ01PxNlv2Tm.SvJhaz/";

  pythonPackagesOverlays = python-final: python-prev: {
    modbus-tk = python-final.callPackage ../../nix/modbus-tk.nix { };
    pymodbus3 = python-final.callPackage ../../nix/pymodbus3.nix { };
  };
  pythonWithOverlay = pkgs.python3.override {
    self = pythonWithOverlay;
    packageOverrides = pkgs.lib.composeManyExtensions [ pythonPackagesOverlays ];
  };
  pythonWithModbus = pythonWithOverlay.withPackages (p: [ p.modbus-tk p.pymodbus3 p.pyserial ]);
in
{
  imports =
    [ ./hardware-configuration.nix
    ];

  networking.hostName = "heizung";

  # Proxmox seems to be a bit capricious w.r.t. order of device on boot
  # and Seabios only sees the first one. Therefore, we play it save and
  # have MBR, ESP and /boot on both of them.
  boot.loader.grub = {
    enable = true;
    devices = [
      "/dev/sda"
      "/dev/sdb"
    ];
    efiSupport = true;
    efiInstallAsRemovable = true;
    mirroredBoots = [
      { path = "/boot";  devices = [ "/dev/sdb" ]; }
      { path = "/boot2"; devices = [ "/dev/sda" ]; }
    ];
  };
  boot.loader.efi = {
    efiSysMountPoint = "/boot";
    canTouchEfiVariables = false;
  };

  networking.useDHCP = true;

  console.keyMap = "us";
  console.font = "Lat2-Terminus16";
  i18n.defaultLocale = "en_US.UTF-8";

  time.timeZone = "Europe/Berlin";

  programs.git.enable = true;

  environment.systemPackages = with pkgs; [
    wget vim neovim git htop tig tmux byobu nano
    usbutils
    platformio
    pythonWithModbus
    picocom
    picotool
  ];

  services.openssh = {
    enable = true;
    settings.PermitRootLogin = "yes";
  };

  networking.firewall.allowedTCPPorts = [
    22  # SSH
  ];

  users = {
    mutableUsers = false;
    users.root = {
      hashedPassword = passwordHash;
      openssh.authorizedKeys.keys = import ./ssh-keys.nix;
    };
    users.bernd = {
      isNormalUser = true;
      uid = 1000;
      hashedPassword = passwordHash;
      extraGroups = [ "wheel" "networkmanager" "dialout" ];
      openssh.authorizedKeys.keys = import ./ssh-keys.nix;
    };
  };

  security.sudo = {
    enable = true;
    wheelNeedsPassword = false;
  };

  services.qemuGuest.enable = true;

  # enable flakes
  nix.settings.experimental-features = [ "nix-command" "flakes" ];

  services.udev.extraRules = ''
    # USB serial
    ACTION=="add", SUBSYSTEM=="tty", SUBSYSTEMS=="usb", ATTRS{idVendor}=="10c4", ATTRS{idProduct}=="ea60", GROUP="dialout", SYMLINK+="ttyUSB-modbus", ENV{ID_MM_DEVICE_IGNORE}="1"

    # Raspberry Pico
    ACTION=="add", SUBSYSTEMS=="usb", ATTRS{idVendor}=="2e8a", GROUP="dialout", ENV{ID_MM_DEVICE_IGNORE}="1"
    ACTION=="add", SUBSYSTEM=="tty", SUBSYSTEMS=="usb", ATTRS{idVendor}=="2e8a", ATTRS{idProduct}=="000a", GROUP="dialout", SYMLINK+="ttyUSB-pico", ENV{ID_MM_DEVICE_IGNORE}="1"
  '';

  # This value determines the NixOS release from which the default
  # settings for stateful data, like file locations and database versions
  # on your system were taken. It‘s perfectly fine and recommended to leave
  # this value at the release version of the first install of this system.
  # Before changing this value read the documentation for this option
  # (e.g. man configuration.nix or on https://nixos.org/nixos/options.html).
  system.stateVersion = "23.11"; # Did you read the comment?
}
